/** 
 *   thermovac - Communication Module for Thermovac Instrument
 *
 *   (c) Copyright 2005 Fernando P. Hauscarriaga 
 *   <fernandoph@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/

#include "vac.h"

/*
 * vac_init - Open device vac
 */

int vac_init(struct vac **vac_conf)
{
  struct termios newtio;
  struct vac *v;
    
  // Allocate vac structure
  v = *vac_conf;
  
  // Initializes devices
  while(v->next != NULL){
    v->fd = open(v->device, O_RDWR);
    if (v->fd < 0)
      return -1;
    else{
      memset(&newtio, 0, sizeof(newtio));
      newtio.c_cflag = (BAUDRATE|CS8|CREAD|CLOCAL);
      newtio.c_iflag &= IXON;
      newtio.c_cc[VMIN]  = 1;
      newtio.c_cc[VTIME] = 0;
      
      tcsetattr(v->fd, TCSANOW, &newtio);
    }
    v = v->next;
  }
  return 0;
}

/*
 * vac_close - Closes devices
 */

int vac_close (struct vac *vac_conf)
{
  struct vac *v;

  v = vac_conf;

  while(v->next != NULL){
    close(v->fd);
    v = v->next;
  }
  return 0;
}

/*
 * vac_sendcmd - Send command to the serial interface
 */

int vac_sendcmd (int fd, char *cmd)
{
  int resul;

  resul = write(fd, cmd, strlen(cmd));
  
  return resul;
}

/*
 * vac_write - Sends a command to the device, checks ACK/NAK and reports
 *                   error conditions in case of NAK
 */ 

int vac_write(int fd, char *cmd)
{
  int resul;
  char eri_cmd[6];
  char buff[21];

  resul = vac_sendcmd (fd, cmd);

  if (resul < 0){
    printf("vac_write failed.\n");
    return -1;
  }
  else {
    if (check_ack(fd) < 0){
      sprintf (eri_cmd, "eri r%c", CR);
      vac_sendcmd (fd, eri_cmd);
      vac_read (fd, buff);
      printf("%s\n", buff);
      return -1;
    }
  }

  return(resul);
}

/*
 * sensor_get - Gets pressure reading sensor
 */

int sensor_get (int fd, char *sensor, double *value, char *units)
{
  char buff[BUFF];
  char request[12];
  char *value_char;
  int i;

  sprintf(request, "mes r %s %c", sensor, CR);

  if (vac_write(fd, request) < 0)
    {
      printf("vac_write failed.\n");
      return -1;
    }
  
  if (vac_read (fd, buff) < 0)
    {
      printf("vac_read failed.\n");
      return -1;
    }
  else 
    {
      value_char = buff+11;
      *value = atof(value_char);
      
      for (i=4; i < 10; i++)
      	units[i-4] = buff[i];
      
      return 0;
    }
}
  
/*
 * vac_read - Reads vac device
 */

int vac_read (int fd, char *recv)
{
  int len = 0;
  int l;

  do{
    l = read (fd, recv+len, 1);
    if(*(recv+len) != LF)
      len += l;
  } while ( len == 0 || recv[len-1] != CR);
  
  return (l);
}

/*
 * check_ack - Checks for ACK char in serial device
 */

int check_ack (int fd)
{
  char recv[BUFF];

  vac_read (fd, recv);
 
  if (recv[0] == ACK)
    return 1;

  else if (recv[0] == NAK)
    return -1;
  return 0;
}

/*
 * count_sensors
 */

int count_sensors (struct vac *vac_conf)
{
  int total_sensors = 0;
  struct vac *v;

  // Allocate vac structure
  v = vac_conf;

  while (v->next != NULL){
    total_sensors = total_sensors + v->sensor_n;
    v = v->next;
  }
  
  return (total_sensors);
}
 
/*
 * get_readings - Gets data from all instruments
 */

int get_readings (struct vac *vac_conf, double *readings)
{
  int i, j;
  struct vac *v;
  char units[6];
  double valor = 0;
 
  // Allocate vac structure
  v = vac_conf;
  
  i = 0;
  while (v->next != NULL){
    for (j = 0; j < v->sensor_n; j++){
      sensor_get (v->fd, v->sensor[j], &valor, units);
      readings[i] = valor;
      i++;
    }
    v = v->next;
  }
  return 0;
}
